@@ -79,6 +79,10 @@ int main(int argc, char **argv)
     QApplication app(argc, argv);
     app.setWindowIcon(QIcon::fromTheme(QStringLiteral("system-file-manager"), app.windowIcon()));
 
+#if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
+    QApplication::setStyle(QStringLiteral("breeze"));
+#endif
+
     KCrash::initialize();
 
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
@@ -245,6 +249,8 @@ int main(int argc, char **argv)
         }
     }
 
+    mainWindow->setSessionAutoSaveEnabled(GeneralSettings::rememberOpenedTabs());
+
 #if HAVE_KUSERFEEDBACK
     auto feedbackProvider = DolphinFeedbackProvider::instance();
     Q_UNUSED(feedbackProvider)
